/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.mete.v1

import de.chaosdorf.mete.model.UserId
import de.chaosdorf.mete.model.UserModel
import de.chaosdorf.mete.util.BigDecimalSerializer
import kotlinx.datetime.Instant
import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable
import java.math.BigDecimal

@Serializable
internal data class UserModelV1(
  @SerialName("id")
  override val userId: UserId,
  @SerialName("name")
  override val name: String,
  @SerialName("email")
  override val email: String,
  @SerialName("balance")
  @Serializable(with = BigDecimalSerializer::class)
  override val balance: BigDecimal,
  @SerialName("active")
  override val active: Boolean,
  @SerialName("audit")
  override val audit: Boolean,
  @SerialName("redirect")
  override val redirect: Boolean,
  @SerialName("created_at")
  val createdAt: Instant,
  @SerialName("updated_at")
  val updatedAt: Instant,
) : UserModel
