/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.sync

import de.chaosdorf.mete.model.MeteApiFactory
import de.chaosdorf.meteroid.model.AccountInfo
import de.chaosdorf.meteroid.model.Drink
import de.chaosdorf.meteroid.model.Server
import de.chaosdorf.meteroid.model.User
import java.math.BigDecimal
import javax.inject.Inject

class SyncManager @Inject constructor(
  private val factory: MeteApiFactory,
  private val userSyncHandler: UserSyncHandler,
  private val drinkSyncHandler: DrinkSyncHandler,
  private val transactionSyncHandler: TransactionSyncHandler
) {
  suspend fun sync(server: Server, user: User?) {
    userSyncHandler.sync(server)
    drinkSyncHandler.sync(server)
    if (user != null) {
      transactionSyncHandler.sync(Pair(server, user.userId))
    }
  }

  suspend fun purchase(account: AccountInfo, drink: Drink) {
    account.user?.let { user ->
      val api = factory.newInstance(account.server.url)
      api.purchase(user.userId, drink.drinkId)
      sync(account.server, user)
    }
  }

  suspend fun deposit(account: AccountInfo, amount: BigDecimal) {
    account.user?.let { user ->
      val api = factory.newInstance(account.server.url)
      api.deposit(user.userId, amount)
      sync(account.server, user)
    }
  }

  suspend fun withdraw(account: AccountInfo, amount: BigDecimal) {
    account.user?.let { user ->
      val api = factory.newInstance(account.server.url)
      api.withdraw(user.userId, amount)
      sync(account.server, user)
    }
  }
}
